@@ -20,6 +20,7 @@ sttype_register_integer(void)
 		"INTEGER",
 		NULL,
 		NULL,
+		NULL,
 		NULL
 	};
 
@@ -27,7 +28,7 @@ sttype_register_integer(void)
 }
 
 /*
- * Editor modelines  -  http://www.wireshark.org/tools/modelines.html
+ * Editor modelines  -  https://www.wireshark.org/tools/modelines.html
  *
  * Local variables:
  * c-basic-offset: 8
