@@ -14,7 +14,6 @@
 #include <glib.h>
 #include <string.h>
 
-#include <config.h>
 #include "ui/iface_toolbar.h"
 
 typedef enum {
@@ -213,7 +212,7 @@ GList * extcap_parse_dlts(gchar *output);
 #endif
 
 /*
- * Editor modelines  -  http://www.wireshark.org/tools/modelines.html
+ * Editor modelines  -  https://www.wireshark.org/tools/modelines.html
  *
  * Local variables:
  * c-basic-offset: 4
